Package: GSgalgoR
Type: Package
Title: An Evolutionary Framework for the Identification and Study of
        Prognostic Gene Expression Signatures in Cancer
Version: 1.22.0
Authors@R: c(person("Martin", "Guerrero", role = c("aut"),
                     email = "mguerrero@conicet-mendoza.gob.ar"),
              person("Carlos", "Catania", role = "cre",email="harpomaxx@gmail.com"))
Description: A multi-objective optimization algorithm for disease sub-type discovery based on a non-dominated sorting genetic algorithm. 
   The 'Galgo' framework combines the advantages of clustering algorithms for grouping heterogeneous 'omics' data and the searching properties of 
   genetic algorithms for feature selection. The algorithm search for the  optimal number of clusters determination considering the features that 
   maximize the survival difference between sub-types while keeping cluster consistency high.
License: MIT + file LICENSE
biocViews: GeneExpression, Transcription, Clustering, Classification,
        Survival
Encoding: UTF-8
Imports: cluster, doParallel, foreach, matchingR, nsga2R, survival,
        proxy, stats, methods,
Suggests: knitr, rmarkdown, ggplot2, BiocStyle, genefu, survcomp,
        Biobase, survminer, breastCancerTRANSBIG, breastCancerUPP,
        iC10TrainingData, pamr, testthat
URL: https://github.com/harpomaxx/GSgalgoR
BugReports: https://github.com/harpomaxx/GSgalgoR/issues
RoxygenNote: 7.1.0
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/GSgalgoR
git_branch: RELEASE_3_23
git_last_commit: 9bd5eb5
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:30:44 UTC; biocbuild
Author: Martin Guerrero [aut],
  Carlos Catania [cre]
Maintainer: Carlos Catania <harpomaxx@gmail.com>
